function [N,Nx,Ny,J] = BaseTri2D(xch,ych,xx,yy);
    
% Minicurso: Metodos de Elementos Finitos para as Eqs de Maxwell
% Saulo P. Oliveira (UFPR), 2012
%
% BaseTri2D.m: funcoes de base lineares (P1) em um triangulo
% Argumentos de entrada: 
%   xch,ych : coordenadas dos pontos de integracao
%   xx,yy   : coordenadas dos vertices do elemento
% Argumentos de saida:
%   N(x,y)  : funcoes de base e derivadas 
%   J       : determinante do jacobiano da transformacao K -> [-1,1]^2

N(1,:) = xch;
N(2,:) = ych;
N(3,:) = 1 - xch - ych;

Nxch = 0.*N;   % inicializa Nxch e Nych
Nych = Nxch;

Nxch(1,:) = 1;
Nxch(2,:) = 0;
Nxch(3,:) = -1;

Nych(1,:) = 0;
Nych(2,:) = 1;
Nych(3,:) = -1;
  
% Calcula as derivadas  dx/dxch, dx/dych, dy/dxch, dy/dych que
% compoem o jacobiano; calcula seu determinante ( J )

dxxch = xx(1)*Nxch(1,:)+xx(2)*Nxch(2,:)+xx(3)*Nxch(3,:);
dyxch = yy(1)*Nxch(1,:)+yy(2)*Nxch(2,:)+yy(3)*Nxch(3,:);
dxych = xx(1)*Nych(1,:)+xx(2)*Nych(2,:)+xx(3)*Nych(3,:);
dyych = yy(1)*Nych(1,:)+yy(2)*Nych(2,:)+yy(3)*Nych(3,:);

J = dxxch.*dyych - dxych.*dyxch;

% Calcula grad N(i,:) (Jac*grad N = grad\hat{N} ) 

for i = 1:3
  Nx(i,:) =  (Nxch(i,:).*dyych-Nych(i,:).*dyxch)./J;             
  Ny(i,:) = -(Nxch(i,:).*dxych-Nych(i,:).*dxxch)./J;
end
