function [N1p,N2p,N1,N2,CurlN,J] = BaseTri2DWit(xch,ych,xx,yy);
    
% Minicurso: Metodos de Elementos Finitos para as Eqs de Maxwell
% Saulo P. Oliveira (UFPR), 2012
%
% funcoes de base de Whitney em um triangulo
% Argumentos de entrada: 
%   xch,ych : coordenadas dos pontos de integracao
%   xx,yy   : coordenadas dos vertices do elemento
% Argumentos de saida:
%   (N1,N2) : funcoes de base
%   curlN   : rotacionais das funcoes de base
%   J       : determinante do jacobiano da transformacao K -> [-1,1]^2

% Calculo da base escalar

N(1,:) = xch;
N(2,:) = ych;
N(3,:) = 1 - xch - ych;

Nxch = 0.*N;   % inicializa Nxch e Nych
Nych = Nxch;

Nxch(1,:) = 1;
Nxch(2,:) = 0;
Nxch(3,:) = -1;

Nych(1,:) = 0;
Nych(2,:) = 1;
Nych(3,:) = -1;

% Calcula as derivadas  dx/dxch, dx/dych, dy/dxch, dy/dych que
% compoem o jacobiano; calcula seu determinante ( J )

dxxch = xx(1)*Nxch(1,:)+xx(2)*Nxch(2,:)+xx(3)*Nxch(3,:);
dyxch = yy(1)*Nxch(1,:)+yy(2)*Nxch(2,:)+yy(3)*Nxch(3,:);
dxych = xx(1)*Nych(1,:)+xx(2)*Nych(2,:)+xx(3)*Nych(3,:);
dyych = yy(1)*Nych(1,:)+yy(2)*Nych(2,:)+yy(3)*Nych(3,:);

J = dxxch.*dyych - dxych.*dyxch;

% Montagem da base vetorial

zero = 0*xch;

N1p = [N(1,:) ; N(2,:) ; N(3,:) ; zero   ; zero   ; zero  ]; 
N2p = [zero   ; zero   ; zero   ; N(1,:) ; N(2,:) ; N(3,:)]; 

nint = length(xch);

% Calcula (N1,N2) e curl(N1,N2) via transformacao do tipo Piola

N1 = N.*Nxch([2,3,1],:) - N([2,3,1],:).*Nxch;
N2 = N.*Nych([2,3,1],:) - N([2,3,1],:).*Nych;

for l = 1:nint
  JInv = [ dyych(l) , -dxych(l) ; -dyxch(l) , dxxch(l) ]/J(l);
  aux = JInv'*[N1(:,l)';N2(:,l)'];
  N1(:,l) = aux(1,:);
  N2(:,l) = aux(2,:);
end

N2xch = Nxch.*Nych([2,3,1],:) - Nxch([2,3,1],:).*Nych;
N1ych = Nych.*Nxch([2,3,1],:) - Nych([2,3,1],:).*Nxch;

for i = 1:3
  CurlN(i,:) = (N2xch(i,:)-N1ych(i,:))./J;
end