% Minicurso: Metodos de Elementos Finitos para as Eqs de Maxwell
% Saulo P. Oliveira (UFPR), 2012
%
% Metodo de elementos finitos para a equacao do Potencial Eletrico
% Elementos triangulares

clear
tic;

% Geometria da malha

tipo = input('Tecle 1 para a malha grossa e 2 para a malha fina: ');

fid     = fopen(['PotEl.' num2str(tipo) '.ele']);
aux     = fscanf(fid,'%i',3);
num_ele = aux(1);
num_vertices_ele = aux(2);

IEN    = zeros(num_vertices_ele,num_ele);
for el = 1:num_ele
  aux  = fscanf(fid,'%i',num_vertices_ele+1);
  IEN(:,el) = aux(2:num_vertices_ele+1)';
end

fid = fopen(['PotEl.' num2str(tipo) '.node']);
aux = fscanf(fid,'%i',4);
num_vertices = aux(1);

coord     = zeros(num_vertices,2);
fronteira = zeros(num_vertices,1);
for j = 1:num_vertices
  aux = fscanf(fid,'%f',4);
  coord(j,:)   = aux(2:3);
  fronteira(j) = aux(4);
end

% Vetor com forca sobre cada vertice

f = zeros(num_vertices,1);  
f(5) =  1.0/(0.01*0.01*0.5);  % intensidade: 1/Area do triangulo adjacente

% Pontos e pesos de integracao

xch  = [0.5,0.0,0.5];
ych  = [0.0,0.5,0.5];
w    = [1,1,1]/6; 
nint = 3;

% Vetor de identificacao e no. de graus de liberdade (variaveis do sistema)

cnt = 0;
ID = zeros(num_vertices,1);	
for j = 1:num_vertices
  if (fronteira(j) ~= 1)
    cnt = cnt + 1;
    ID(j) = cnt;
  end
end
dof = cnt;

for j = 1:num_vertices  % cria "lixeira" para vertices sem graus de liberdade
  if(ID(j)==0)
    ID(j) = dof + 1;
  end
end

% Algoritmo de montagem

K = sparse(dof+1,dof+1);
b = sparse(dof+1,1);

for el = 1:num_ele
  
% Inicializa matriz e vetor locais

  Ae = zeros(num_vertices_ele,num_vertices_ele);
  be = zeros(num_vertices_ele,num_vertices_ele);
   
  % coordenadas dos vertices do elemento
  
  ive = IEN(:,el);   % indices dos vertices do elemento
  xx  = coord(ive,1);  
  yy  = coord(ive,2);
  
  % Calculo das matrizes locais por quadratura Gaussiana
  
  [N,Nx,Ny,J] = BaseTri2D(xch,ych,xx,yy);

  for j = 1:num_vertices_ele 
    for l = 1:num_vertices_ele
      Ae(j,l) = sum( J.*w.*(Nx(l,:).*Nx(j,:)+Ny(l,:).*Ny(j,:)) );
      be(j,l) = sum( J.*w.*(N(l,:).*N(j,:)) );
    end
  end
  
  % Adiciona Ae em K

  II = ID(ive);
  K(II,II) = K(II,II) + Ae;
  
  % Montagem do lado direito do sistema (a fonte e' interpolada)
  
  ff = f(ive);
  b(II) = b(II) + be*ff;

end

% solucao do sistema linear 

disp(['Tempo de montagem: ',num2str(toc)]);
tic;
uh = K(1:dof,1:dof)\b(1:dof);
uh(dof+1) = 0;
disp(['Tempo de solucao do sistema: ',num2str(toc)]);

% Prepara resultado para exibicao

ul = zeros(3,num_ele);
for el = 1:num_ele
  ul(:,el) = uh(ID(IEN(:,el)));
end

ul(4,:)  =  ul(3,:); % esta definicao permite o uso das
IEN(4,:) = IEN(3,:); % rotinas graficas para quadrilateros